import os
from pathlib import Path

import pytest


def pytest_configure(config):
    # required to make sure files are created in the same directory as the test file
    os.chdir(Path(__file__).parent.resolve())


@pytest.fixture(autouse=True)
def setup_and_teardown():
    # Setup code
    yield
    # Teardown code
    for path in Path(__file__).resolve().parent.iterdir():
        print(path)
        if path.name.endswith(".gitignore"):
            path.unlink()
        elif path.name.endswith(".prettierignore"):
            path.unlink()



from __future__ import annotations

from pathlib import Path

import pytest
import typer

from easyignore.main import (
    get_file_type_links,
    get_gitignore,
    get_gitignores,
    main,
    validate_gitignores,
)


@pytest.fixture()
def example_gitignore():
    path = Path("testdata/.gitignore")
    return path.read_text()


def test_get_gitignore(example_gitignore):
    languages = ["python"]
    gitignore = get_gitignore(languages)
    assert gitignore == example_gitignore


def test_get_gitignores():
    gitignores = get_gitignores()
    assert isinstance(gitignores, list)
    assert len(gitignores) > 0
    assert "python" in gitignores
    assert "node" in gitignores
    assert "react" in gitignores
    assert "c++" in gitignores
    assert "rust" in gitignores
    assert "csharp" in gitignores


def test_validate_gitignores():
    languages = ["python", "node", "react"]
    invalid_lang = "invalidlang"
    assert validate_gitignores(languages) == languages
    with pytest.raises(typer.BadParameter):
        validate_gitignores([invalid_lang])


def test_get_file_type_links():
    language = ["python"]
    assert get_file_type_links(language) == "[bold][link=https://gitignore.io/api/python]python[/link][/bold]"
    languages = ["python", "node"]
    assert (
        get_file_type_links(languages)
        == "[bold][link=https://gitignore.io/api/python]python[/link][/bold] and [bold][link=https:///gitignore.io/api/node]node[/link][/bold]"
    )
    languages = ["python", "node", "react"]
    assert (
        get_file_type_links(languages)
        == "[link=https://gitignore.io/api/python]python[/link], [link=https://gitignore.io/api/node]node[/link], and [bold][link=https://gitignore.io/api/react]react[/link][/bold]"
    )


def test_main(example_gitignore):
    languages = ["python"]
    gitignore = main(languages)
    gitignore = Path(".").joinpath(".gitignore").read_text()
    assert gitignore == example_gitignore



import re
from difflib import get_close_matches
from pathlib import Path
from typing import Annotated

import requests
import typer
import typer.completion
from rich import print as rprint
from rich.columns import Columns
from rich.prompt import Prompt

try:
    import shellingham
except ImportError:
    shellingham = None
    rprint("[red]Shell completion requires the shellingham package. Please install it with `uv add shellingham`.[/red]")

app = typer.Typer(
    context_settings={
        "help_option_names": ["-h", "--help"],
    },
    rich_markup_mode="rich",
    rich_help_panel=True,
    no_args_is_help=True,
    add_completion=False,
    help="Create a .gitignore or .prettierignore file for over 500 languages and frameworks. Currently, the source for the ignore files is https://gitignore.io but this may change in the future to ensure the most up-to-date ignore files are used (see also https://github.com/toptal/gitignore.io/issues/650)",
)
typer.completion.completion_init()

# app_completion = typer.Typer(help="Generate and install completion scripts.", hidden=True)
# app.add_typer(app_completion, name="completion")


def get_file_type_links(languages: list[str]) -> str:
    if len(languages) == 1:
        return f"[bold][link=https://gitignore.io/api/{languages[0]}]{languages[0]}[/link][/bold]"
    elif len(languages) == 2:
        return f"[bold][link=https://gitignore.io/api/{languages[0]}]{languages[0]}[/link][/bold] and [bold][link=https:///gitignore.io/api/{languages[1]}]{languages[1]}[/link][/bold]"
    elif len(languages) > 2:
        languages_str = ", ".join([f"[link=https://gitignore.io/api/{ft}]{ft}[/link]" for ft in languages[:-1]])
        return (
            f"{languages_str}, and [bold][link=https://gitignore.io/api/{languages[-1]}]{languages[-1]}[/link][/bold]"
        )


def get_gitignore(languages: list[str]) -> str:
    languages = ",".join([f"{ft}" for ft in languages])
    response = requests.get(f"https://gitignore.io/api/{languages}")
    if response.status_code == 200:
        return f"# Created with easyignore\n{response.text}"
    else:
        rprint(
            f"[red]Failed to fetch gitignore for {languages}. Check that {languages} is valid and that you are connected to the internet and try again.[/red]"
        )
        raise typer.Exit(code=1)


def get_gitignores() -> list[str]:
    response = requests.get("https://gitignore.io/api/list")
    if response.status_code == 200:
        return re.split("\n|,", response.text)
    else:
        rprint("[red]Failed to fetch gitignore list. Check your internet connection and try again.[/red]")
        raise typer.Exit(code=1)


def complete_gitignores() -> list[str]:
    return get_gitignores()


def validate_gitignores(value: list[str]) -> str:
    for v in value:
        if v not in complete_gitignores():
            best_matches = get_close_matches(v, complete_gitignores(), n=5, cutoff=0)
            if best_matches:
                raise typer.BadParameter(f"Invalid language: {v}. Perhaps you meant one of: {', '.join(best_matches)}.")
            else:
                raise typer.BadParameter(f"Invalid language: {v}. No close matches found.")
    return value


def list_gitignores(value: bool) -> None:
    """
    List available languages/frameworks available from gitignore.io
    """
    if not value:
        return
    git_ignores = get_gitignores()
    git_ignores = [f"[blue][link=https://gitignore.io/api/{g}]{g}[/link][/blue]" for g in git_ignores]
    git_ignores = Columns(git_ignores, equal=True, expand=True)
    rprint("[bold]Available languages/frameworks for .gitignore:[/bold]")
    rprint(git_ignores)
    raise typer.Exit(code=0)


def show_completion(ctx: typer.Context, value: bool) -> None:
    if value:
        if shellingham is None:
            raise typer.BadParameter(
                "Shell completion requires the shellingham package. Please install it with `uv add shellingham`."
            )
        shell, _ = shellingham.detect_shell()
        typer.completion.show_callback(ctx, None, shell)
        raise typer.Exit(code=0)


def install_completion(ctx: typer.Context, value: bool) -> None:
    if value:
        if shellingham is None:
            raise typer.BadParameter(
                "Shell completion requires the shellingham package. Please install it with `uv add shellingham`."
            )
        shell, _ = shellingham.detect_shell()
        typer.completion.install_callback(ctx, None, shell)
        raise typer.Exit(code=0)


@app.command(no_args_is_help=True)
def main(
    languages: Annotated[
        list[str],
        typer.Argument(
            help="language/framework for .gitignore (enter as many as you like)",
            autocompletion=get_gitignores,
            # shell_complete=get_gitignores,
            callback=validate_gitignores,
            is_eager=True,
        ),
    ],
    path: Annotated[
        Path | None,
        # this is not working yet - see
        typer.Option(
            "--path",
            "-p",
            help="path to .gitignore file [default: current directory]",
            file_okay=False,
            dir_okay=True,
            exists=True,
            is_eager=True,
        ),
    ] = Path.cwd(),  # noqa: B008  # function-call-in-default-argument
    append: Annotated[
        bool | None,
        typer.Option(
            "--append",
            "-a",
            help="append to existing .gitignore file",
            show_default=True,
            is_eager=True,
        ),
    ] = False,
    overwrite: Annotated[
        bool | None,
        typer.Option(
            "--overwrite",
            "-o",
            help="overwrite existing .gitignore file",
            show_default=True,
            is_eager=True,
        ),
    ] = False,
    prettier: Annotated[
        bool | None,
        typer.Option(
            "--prettier",
            "-r",
            help="save as .prettierignore",
            show_default=True,
            is_eager=True,
        ),
    ] = False,
    list_gitignores: Annotated[
        bool | None,
        typer.Option(
            "--list",
            "-l",
            help="list available languages/frameworks for .gitignore",
            is_eager=True,
            callback=list_gitignores,
        ),
    ] = False,
    # custom handling of install completion and show completion
    # allows no arguments to be passed when installing or showing completion scripts
    install_completion: Annotated[
        bool | None,
        typer.Option(
            "--install-completion",
            "-i",
            help="install shell completion for easyignore",
            callback=install_completion,
            is_eager=True,
        ),
    ] = False,
    show_completion: Annotated[
        bool | None,
        typer.Option(
            "--show-completion",
            "-s",
            help="show shell completion for easyignore",
            callback=show_completion,
            is_eager=True,
        ),
    ] = False,
) -> None:
    """
    Create a .gitignore (or .prettierignore with --prettier) file for over 500 languages and frameworks.
    Currently, the source for the ignore files is https://gitignore.io but this may change in the future
    to ensure the most up-to-date ignore files are used (see also https://github.com/toptal/gitignore.io/issues/650).
    """
    # Consider using https://donotcommit.com/api as a source instead of gitignore.io once they expand available languages
    if append and overwrite:
        raise typer.BadParameter(
            "Cannot use both append and overwrite options at the same time.",
            param_hint=["append", "overwrite"],
        )
    if path.is_dir():
        if prettier:
            path = path / ".prettierignore"
        else:
            path = path / ".gitignore"
    else:
        # this should be caught by the file_okay=False option
        raise typer.BadParameter(
            "Path must be a directory. Please provide a directory path to create the .gitignore file."
        )
    if path.exists():
        if not append and not overwrite:
            a_o_c = Prompt.ask(
                f"{path} already exists. Do you want to overwrite, append, or cancel? (o/a/c)",
            )
            if a_o_c == "a":
                append = True
                overwrite = False
            elif a_o_c == "o":
                append = False
                overwrite = True
            else:
                raise typer.Abort()
        if append and not overwrite:
            with open(path, "a+") as f:
                f.write(f"\n\n{get_gitignore(languages)}")
            rprint(
                f"[green]Appended {get_file_type_links(languages)} {'prettierignore' if prettier else 'gitignore'} to existing file at {path}[/green]"
            )
        elif overwrite and not append:
            with open(path, "w") as f:
                f.write(get_gitignore(languages))
            rprint(
                f"[green]Overwrote existing file with {get_file_type_links(languages)} {'prettierignore' if prettier else 'gitignore'} at {path}[/green]"
            )
        else:
            # this should be caught by the append and overwrite check above
            raise typer.BadParameter(
                "Cannot use both append and overwrite options at the same time.",
                param_hint=["append", "overwrite"],
            )
    else:
        with open(path, "w") as f:
            f.write(get_gitignore(languages))
        rprint(
            f"[green]Created {get_file_type_links(languages)} {'prettierignore' if prettier else 'gitignore'} at {path}[/green]"
        )
    typer.Exit(code=0)


if __name__ == "__main__":
    app()



from __future__ import annotations

from pathlib import Path

import pytest
from typer.testing import CliRunner

from easyignore.main import app

runner = CliRunner()


@pytest.fixture()
def path():
    return Path(__file__).parent.resolve()


def test_app():
    result = runner.invoke(app, ["python", "-o"])
    assert result.exit_code == 0
    assert Path(".gitignore").exists()


def test_multiple_args():
    result = runner.invoke(app, ["python", "node", "-o"])
    assert result.exit_code == 0
    assert Path(".gitignore").exists()
    result = runner.invoke(app, ["python", "node", "react", "-o"])
    assert result.exit_code == 0
    result = runner.invoke(app, ["python", "c++", "rust", "csharp", "-o"])
    assert result.exit_code == 0


def test_invalid_args():
    result = runner.invoke(app, ["python", "node", "invalidlang"])
    assert result.exit_code == 2
    # test for notification of invalid language
    assert "Invalid value for 'LANGUAGES...': Invalid language: invalidlang" in result.stdout
    # test for generating close matches
    assert "pythonvanilla, leiningen, xilinx, vivado, vaadin" in result.stdout
    assert not Path(".gitignore").exists()


def test_help():
    result = runner.invoke(app, [])
    assert result.exit_code == 0
    result = runner.invoke(app, ["--help"])
    assert result.exit_code == 0
    result = runner.invoke(app, ["-h"])
    assert result.exit_code == 0


def test_prettier():
    result = runner.invoke(
        app,
        [
            "python",
            "-r",
        ],
    )
    assert result.exit_code == 0
    assert Path(".prettierignore").exists()
    result = runner.invoke(app, ["python", "--overwrite", "--prettier"])
    assert result.exit_code == 0
    result = runner.invoke(app, ["python", "--append", "--prettier"])
    assert result.exit_code == 0


def test_append():
    result = runner.invoke(app, ["python", "-a"])
    assert result.exit_code == 0
    assert Path(".gitignore").exists()
    result = runner.invoke(app, ["python", "--append"])
    assert result.exit_code == 0
    result = runner.invoke(app, ["python", "-o", "-a"])
    assert result.exit_code == 2
    assert "Invalid value for 'append' / 'overwrite'" in result.stdout


def test_overwrite():
    result = runner.invoke(app, ["python", "-o"])
    assert result.exit_code == 0
    assert Path(".gitignore").exists()
    result = runner.invoke(app, ["python", "--overwrite"])
    assert result.exit_code == 0
    result = runner.invoke(app, ["python", "-a", "-o"])
    assert result.exit_code == 2
    assert "Invalid value for 'append' / 'overwrite'" in result.stdout


def test_list():
    result = runner.invoke(app, ["python", "-l"])
    assert result.exit_code == 0
    result = runner.invoke(app, ["python", "--list"])
    assert result.exit_code == 0
    result = runner.invoke(app, ["python", "-o", "-l"])
    assert result.exit_code == 0
    result = runner.invoke(app, ["python", "--overwrite", "--list"])
    assert result.exit_code == 0
    assert not Path(".gitignore").exists()


def test_path():
    path = str(Path(__file__).resolve().parent)
    # first test with no overwrite - should work since the file doesn't exist
    result = runner.invoke(app, ["python", "-p", path])
    assert result.exit_code == 0
    assert Path(".gitignore").exists()
    # subsequent tests need to overwrite
    result = runner.invoke(app, ["python", "-o", "-p", path])
    assert result.exit_code == 0
    result = runner.invoke(app, ["python", "--overwrite", "--path", path])
    assert result.exit_code == 0


def test_no_options():
    result = runner.invoke(app, ["python"])
    assert result.exit_code == 0
    result = runner.invoke(app, ["python"], input="a")
    assert result.exit_code == 0
    result = runner.invoke(app, ["python"], input="o")
    assert result.exit_code == 0
    result = runner.invoke(app, ["python"], input="c")
    assert result.exit_code == 1
